/*jshint esversion: 6 */

// TODO: alterar para tratar de forma diferente diferentes claim types

const request = require('request-promise');
const http = require('http');
const https = require('https');

module.exports = function(game, options){
  return new MailboxHandler(game, options);
};

/**
 * Create a Mailbox handler
 * @param {string} game - The game's name to send to Mailbox
 * @param {Object} options - Mailbox options used to make HTTP requests
 * @param {string} options.url - The Mailbox url, default is http://mailbox.mailbox
 * @param {string} options.user - User basic auth
 * @param {string} options.pass - Pass basic auth
 * @param {number} options.timeout - Request timeout in ms, default is 1000ms
 */
const MailboxHandler = function(game, options) {
  this.game = game;
  this.options = options;
  this.options.url = this.options.url || 'http://mailbox.mailbox';
  this.options.timeout = this.options.timeout || 1000;

  const maxSockets = options.maxSockets || 50;
  const agentOptions = {
    keepAlive: true,
    maxSockets,
  };

  if (this.options.url.indexOf('https') !== -1) {
    this.keepAliveAgent = new https.Agent(agentOptions);
  } else {
    this.keepAliveAgent = new http.Agent(agentOptions);
  }
};

const buildUrl = function(url, game, pathname) {
  return `${url}/games/${game}/${pathname}`;
};

const buildAddClaimUrl = function(url, game, params) {
  return buildUrl(url, game, `players/${params.playerId}/claims`);
};

const buildPlayerClaimsUrl = function(url, game, params) {
  return buildUrl(url, game, `players/${params.playerId}/claims?available=true`);
};

const buildSetClaimUrl = function(url, game, params) {
  return buildUrl(url, game, `players/${params.playerId}/claims/${params.claimId}/claim`);
};

const buildGetClaimUrl = function(url, game, params) {
  return buildUrl(url, game, `players/${params.playerId}/claims/${params.claimId}`);
};

const buildAllPlayerClaimsUrl = function(url, game, params) {
  return buildUrl(url, game, `players/${params.playerId}/claims`);
};

const setClaimedOnMailbox = function(self, params) {
  const url = buildSetClaimUrl(self.options.url, self.game, params);
  const options = {
    url: url,
    method: 'PUT',
    headers: { 'Content-Type': 'application/json' },
    auth: {user: self.options.user, pass: self.options.pass},
    timeout: self.options.timeout,
    agent: self.keepAliveAgent,
    json: true,
  };
  return request(options);
};

const shouldClaimOnGamesBackend = function(claim) {
  return claim.type === "claim:resource"
}

/**
 * Add claim to player adds resources and message as a claim to player
 * @param {string} playerId - Player's id
 * @param {Object} - Resources to give to player
 * @param {string} message - Message to show to player
 * @returns {Promise} Request's promise
 */
MailboxHandler.prototype.addClaim = function (playerId, resources, message) {
  const url = buildAddClaimUrl(this.options.url, this.game, {
    playerId: playerId,
  });
  const body = { resources, message }
  const options = {
    url: url,
    method: 'POST',
    headers: { 'Content-Type': 'application/json' },
    auth: {user: this.options.user, pass: this.options.pass},
    timeout: this.options.timeout,
    agent: this.keepAliveAgent,
    json: true,
    body,
  };
  return request(options);
};

/**
 * Get player available claims
 * @param {string} playerId - Player's id
 * @returns {Promise} Request's promise
 */
MailboxHandler.prototype.getPlayerClaims = function (playerId) {
  const url = buildPlayerClaimsUrl(this.options.url, this.game, {
    playerId: playerId,
  });
  const options = {
    url: url,
    method: 'GET',
    headers: { 'Content-Type': 'application/json' },
    auth: {user: this.options.user, pass: this.options.pass},
    timeout: this.options.timeout,
    agent: this.keepAliveAgent,
    json: true,
  };
  return request(options);
}
/**
 * Get all player claims, including claimed and not claimed
 * @param {string} playerId - Player's id
 * @returns {Promise} Request's promise
 */
MailboxHandler.prototype.getAllPlayerClaims = function (playerId) {
  const url = buildAllPlayerClaimsUrl(this.options.url, this.game, {
    playerId: playerId,
  });
  const options = {
    url: url,
    method: 'GET',
    headers: { 'Content-Type': 'application/json' },
    auth: {user: this.options.user, pass: this.options.pass},
    timeout: this.options.timeout,
    agent: this.keepAliveAgent,
    json: true,
  };
  return request(options);
};

/**
 * Get specific claim
 * @param {string} playerId - Player's id
 * @param {string} claimId - id from claim player is claiming
 * @returns {Promise} Request's promise
 */
MailboxHandler.prototype.getClaim = function (playerId, claimId) {
  const url = buildGetClaimUrl(this.options.url, this.game, {
    playerId: playerId,
    claimId: claimId,
  });
  const options = {
    url: url,
    method: 'GET',
    headers: { 'Content-Type': 'application/json' },
    auth: {user: this.options.user, pass: this.options.pass},
    timeout: this.options.timeout,
    agent: this.keepAliveAgent,
    json: true,
  };
  return request(options);
};

/**
 * Updates claim on game's database and send request to update on Mailbox API
 * @param {string} playerId - Player's id
 * @param {string} claimId - Player's claim id got from getPlayerClaims
 * @param {Mailbox~alreadyClaimedOnGameFunc} alreadyClaimedOnGameFunc - a function that checks if claimId exists on 
 *        the game's database, more information below.
 * @param {Mailbox~giveResourceAndAddClaimIdFunc} giveResourceAndAddClaimIdFunc - a function that adds the resources
 *        to the player, more information below.
 */
MailboxHandler.prototype.doClaim = function* (
  playerId, claimId,
  alreadyClaimedOnGameFunc, giveResourceAndAddClaimIdFunc
) {
  let response;
  try {
    response = yield this.getClaim(playerId, claimId);
  } catch (e) {
    return {success: false, error: e};
  }

  // no response or not a claim field, so some error happened
  if (!response || !response.claim) {
    return response;
  }

  // the resource was already claimed in the mailbox, so return
  if (response.claim.claimed) {
    return {success: false, error: 'resource was already claimed on Mailbox'};
  }

  if (shouldClaimOnGamesBackend(response.claim)) {
    let alreadyClaimed;
    try {
      alreadyClaimed = yield* alreadyClaimedOnGameFunc(response.claim, playerId);
    } catch (e) {
      return {success: false, error: e};
    }

    // wasn't claimed on game, so claim it
    if (!alreadyClaimed) {
      try {
        yield* giveResourceAndAddClaimIdFunc(response.claim, playerId);
      } catch (e) {
        return {success: false, error: e};
      }
    }
  }

  // send to Mailbox if was already claimed on the game or not, because either way the mailbox
  // has claimed=false to this resource
  return yield setClaimedOnMailbox(this, {
    playerId: playerId,
    claimId: claimId,
  });
};

/**
 * A promise with the claim status: claimed or not
 *
 * @promise AlreadyClaimedResult
 * @fulfill {boolean} true if the resource was already claimed, false otherwise
 * @reject {Object|string} error if some occurred during select claim on database
 */

/**
 * This function checks if a resource was already claimed by the player,
 * In other words, checks if there is a claimId for playerId on the claims table.
 *
 * @callback Mailbox~alreadyClaimedOnGameFunc
 *
 * @param {Object} claim - object containing claim's information
 * @param {string} claim.id - claim's id
 * @param {Object} player - object containing player's information
 * @returns {AlreadyClaimedResult} alreadyClaimedResult - a promise with information about the claim
 */

/**
 * This is the function called to add the resource to the player.
 * It must be executed inside a transaction:
 * 1) add the resource to the player
 * 2) add claimId to claims table
 * 3) if one of them failed, rollback both and return error != null
 *
 * @callback Mailbox~giveResourceAndAddClaimIdFunc
 *
 * @param {Object} claim - object with informations about claim
 * @param {string} claim.id - claim's id
 * @param {Object} player - player object
 * @param {string} player.id - player's id
 * @returns {Object|null} Returns null if successful, or an object with error details if failed
 */
