/*jshint esversion: 6 */

const assert = require('assert');
const Mailbox = require('./mailbox.js');
const request = require('request-promise');
require('co-mocha');

function getUrl(url, gameId, playerId) {
  return `${url}/games/${gameId}/players/${playerId}/claims`;
}

describe('MailboxHandler', function() {
  const url = 'http://localhost:8080';
  const gameId = 'game';
  const player = { id: '580ef5d0-9bd0-4b57-b4f6-f36692f936b2' };
  const claim = {
    resources: {gems: 100},
    message: 'this is a message'
  };
  const mailbox = Mailbox('game', { url: url, user: '', pass: '' });

  describe('getPlayerClaims', function() {
    it('should get player claims', function*() {
      yield request.post({
        url: getUrl(url, gameId, player.id),
        method: 'POST',
        body: claim,
        headers: { 'Content-Type': 'application/json' },
        json: true,
      });

      const claims = yield mailbox.getPlayerClaims(player.id);
      const returnedClaim = claims.claims[0];

      assert.equal(returnedClaim.game, gameId);
      assert.equal(returnedClaim.playerId, player.id);
      assert.equal(returnedClaim.message, claim.message);
      assert.deepEqual(returnedClaim.resources, claim.resources);
      assert.equal(returnedClaim.claimed, false);
      assert.ok(returnedClaim.createdAt);
      assert.ok(returnedClaim.updatedAt, false);

      yield request.delete(`${getUrl(url, gameId, player.id)}/${returnedClaim.id}`);
    });
  });

  describe('getClaim', function() {
    it('should get claim', function*() {
      const newClaims = yield request.post({
        url: getUrl(url, gameId, player.id),
        method: 'POST',
        body: claim,
        headers: { 'Content-Type': 'application/json' },
        json: true,
      });

      const returnedClaim = newClaims.successfulClaims[0];
      const gotClaim = yield mailbox.getClaim(player.id, returnedClaim.id);

      assert.equal(returnedClaim.id, gotClaim.claim.id);
      assert.equal(returnedClaim.game, gotClaim.claim.game);
      assert.equal(returnedClaim.playerId, gotClaim.claim.playerId);
      assert.equal(returnedClaim.message, claim.message);
      assert.deepEqual(returnedClaim.resources, claim.resources);
      assert.equal(returnedClaim.claimed, false);
      assert.ok(returnedClaim.createdAt);
      assert.ok(returnedClaim.updatedAt, false);

      yield request.delete(`${getUrl(url, gameId, player.id)}/${returnedClaim.id}`);
    });
  });

  describe('doClaim', function() {
    it('should return if already claimed on game', function*() {
      const newClaims = yield request.post({
        url: getUrl(url, gameId, player.id),
        method: 'POST',
        body: claim,
        headers: { 'Content-Type': 'application/json' },
        json: true,
      });

      const alreadyClaimedOnGameFunc = function(claim) {
        return new Promise(function(resolve) {
          resolve(true);
        });
      };

      const returnedClaim = newClaims.successfulClaims[0];
      const claims = yield mailbox.doClaim(
        player.id, returnedClaim.id,
        alreadyClaimedOnGameFunc
      );

      const gotClaim = yield mailbox.getClaim(player.id, returnedClaim.id);

      assert.equal(gotClaim.claim.claimed, true);

      yield request.delete(`${getUrl(url, gameId, player.id)}/${returnedClaim.id}`);
    });

    it('should continue if not claimed on game', function*() {
      const newClaims = yield request.post({
        url: getUrl(url, gameId, player.id),
        method: 'POST',
        body: claim,
        headers: { 'Content-Type': 'application/json' },
        json: true,
      });

      const returnedClaim = newClaims.successfulClaims[0];

      const alreadyClaimedOnGameFunc = function(claim) {
        return new Promise(function(resolve) {
          resolve(false);
        });
      };

      const giveResourceAndAddClaimIdFunc = function(claim, player) {
        assert.deepEqual(claim, returnedClaim);
        return new Promise(function(resolve) {
          resolve();
        });
      };

      const claims = yield mailbox.doClaim(
        player.id, returnedClaim.id,
        alreadyClaimedOnGameFunc,
        giveResourceAndAddClaimIdFunc
      );

      const gotClaim = yield mailbox.getClaim(player.id, returnedClaim.id);

      assert.equal(gotClaim.claim.claimed, true);

      yield request.delete(`${getUrl(url, gameId, player.id)}/${returnedClaim.id}`);
    });
  });
});
